<?php
namespace OrganizationsManagement\Factory;

use Teknisa\Model\Geral\Entities\Organizacao as EntidadeOrganizacao;
use OrganizationsManagement\Util;
use Setforms2\Shared\ORM\DateTime as SetformsDateTime;

class Organizacao {
    public function __construct(){}

    public function atualiza( EntidadeOrganizacao $entity, $companyName, $fantasyName, $type, $legalInscriptionNumber,
                              $isActive, $dsemailoper, $nmoperador, $nmacesso,
                              $streetName, $number, $complement, $cep, $countryId, $stateSymbol, $cityId, $neighborhood,
                              $cityInscription, $stateInscription, $nrParcNegocio = null, $idOrgSaas, $prepStatus, $nrOrgPrincipal) {
        if($type == EntidadeOrganizacao::PESSOA_JURIDICA || $type == EntidadeOrganizacao::PESSOA_FISICA) {
            $legalInscriptionNumber = Util\Utilities::removeCaracteresEspeciais($legalInscriptionNumber);
        }
        $entity->setDtultatu(new SetformsDateTime());

        $entity->setNmrazsocialorg($companyName);
        $entity->setNmorganizacao($fantasyName);
        $entity->setIdtipoinscrorg($type);
        $entity->setNrinsjurempr($legalInscriptionNumber);
        $entity->setIdativo($isActive);
        $entity->setNrparcnegocio($nrParcNegocio);

        $entity->setNmacesso($nmacesso);
        $entity->setDsemailoper($dsemailoper);
        $entity->setNmoperador($nmoperador);

        $entity->setDslogradempr($streetName);
        $entity->setNrlogradempr($number);
        $entity->setDscmplogempr($complement);
        $entity->setNrcepempr($cep);
        $entity->setCdpais($countryId);
        $entity->setSgestado($stateSymbol);
        $entity->setCdmunicipio($cityId);
        $entity->setNmbairempr($neighborhood);

        $entity->setCdinscmuni($cityInscription);
        $entity->setNrinscesta($stateInscription);

        $entity->setIdorgsaas($idOrgSaas);
        $entity->setNrorgprincipal($nrOrgPrincipal);

        if(in_array($prepStatus, EntidadeOrganizacao::getStatusIniciais())) {
            if($entity->isSecundaria()) {
                $entity->setPrepstatus(EntidadeOrganizacao::PREPARACAO_PENDENTE_SECUNDARIA);
            } else if($entity->isSaas()) {
                $entity->setPrepstatus(EntidadeOrganizacao::GERAR_ESTRUTURA);
            } else {
                $entity->setPrepstatus(EntidadeOrganizacao::PREPARACAO_PENDENTE_LOCAL_HOSTING);
            }
        } else {
            $entity->setPrepstatus($prepStatus);
        }

        return $entity;
    }

    public function cria($id, $companyName, $fantasyName, $type, $inscriptionNumber, $legalInscriptionNumber, $isActive,
                         $dsemailoper, $nmoperador, $nmacesso, $streetName, $number, $complement, $cep, $countryId,
                         $stateSymbol, $cityId, $neighborhood, $cityInscription, $stateInscription, $nrParcNegocio = null, $idOrgSaas, $nrOrgPrincipal) {
        if($type == EntidadeOrganizacao::PESSOA_JURIDICA || $type == EntidadeOrganizacao::PESSOA_FISICA) {
            $legalInscriptionNumber = Util\Utilities::removeCaracteresEspeciais($legalInscriptionNumber);
        }
        $entity = new EntidadeOrganizacao();

        $entity->setNrorg($id);
        $entity->setNrinscricaoorg($inscriptionNumber);

        $entity->setIdtipoinscrorg($type);
        $entity->setNmrazsocialorg($companyName);
        $entity->setNmorganizacao($fantasyName);
        $entity->setIdativo($isActive);
        $entity->setNrinsjurempr($legalInscriptionNumber);
        $entity->setNrparcnegocio($nrParcNegocio);

        $entity->setNmacesso($nmacesso);
        $entity->setDsemailoper($dsemailoper);
        $entity->setNmoperador($nmoperador);

        $entity->setDslogradempr($streetName);
        $entity->setNrlogradempr($number);
        $entity->setDscmplogempr($complement);
        $entity->setNrcepempr($cep);
        $entity->setCdpais($countryId);
        $entity->setSgestado($stateSymbol);
        $entity->setCdmunicipio($cityId);
        $entity->setNmbairempr($neighborhood);

        $entity->setCdinscmuni($cityInscription);
        $entity->setNrinscesta($stateInscription);
        
        $entity->setIdorgsaas($idOrgSaas);
        $entity->setNrorgprincipal($nrOrgPrincipal);

        if($entity->isSecundaria()) {
            $entity->setPrepstatus(EntidadeOrganizacao::PREPARACAO_PENDENTE_SECUNDARIA);
        } else if($entity->isSaas()) {
            $entity->setPrepstatus(EntidadeOrganizacao::GERAR_ESTRUTURA);
        } else {
            $entity->setPrepstatus(EntidadeOrganizacao::PREPARACAO_PENDENTE_LOCAL_HOSTING);
        }

        $entity->setDtinclusao(new SetformsDateTime());
        $entity->setDtultatu(new SetformsDateTime());

        return $entity;
    }

    public function defineStatusSolicitado(EntidadeOrganizacao $e) {
        Util\InstanceProvider::getRepository()->getOrganization()->saveOrganizationPrepStatusBkp($e->getPrepstatus(), $e->getNrorg());
        $e->setPrepstatus(EntidadeOrganizacao::GERACAO_ESTRUTURA_SOLICITADA);
        return $e;
    }

    public function defineStatusProntoParaPreparacao(EntidadeOrganizacao $e) {
        if($e->getPrepstatus() != EntidadeOrganizacao::GERACAO_ESTRUTURA_SOLICITADA) {
            Util\InstanceProvider::getRepository()->getOrganization()->saveOrganizationPrepStatusBkp($e->getPrepstatus(), $e->getNrorg());
        }
        $e->setPrepstatus(EntidadeOrganizacao::GERAR_DADOS_INICIAIS);
        return $e;
    }

    public function defineStatusPreparado(EntidadeOrganizacao $e) {
        if($e->isSecundaria()) {
            $prepStatus = EntidadeOrganizacao::BASE_PREPARADA_SECUNDARIA;
        } else if($e->isSaas()) {
            $prepStatus = EntidadeOrganizacao::BASE_PREPARADA;
        } else {
            $prepStatus = EntidadeOrganizacao::BASE_PREPARADA_LOCAL_HOSTING;
        }
        Util\InstanceProvider::getRepository()->getOrganization()->saveOrganizationPrepStatusBkp($e->getPrepstatus(), $e->getNrorg());
        $e->setPrepstatus($prepStatus);
        return $e;
    }

    public function defineStatusPreparadoZmart(EntidadeOrganizacao $e) {
        if($e->isSecundaria()) {
            $prepStatus = EntidadeOrganizacao::BASE_PREPARADA_ZMART_SECUNDARIA;
        } else if($e->isSaas()) {
            $prepStatus = EntidadeOrganizacao::BASE_PREPARADA_ZMART;
        } else {
            $prepStatus = EntidadeOrganizacao::BASE_PREPARADA_ZMART_LOCAL_HOSTING;
        }
        Util\InstanceProvider::getRepository()->getOrganization()->saveOrganizationPrepStatusBkp($e->getPrepstatus(), $e->getNrorg());
        $e->setPrepstatus($prepStatus);
        return $e;
    }

    public function defineStatusInativo(EntidadeOrganizacao $e) {
        if($e->isSecundaria()) {
            $prepStatus = EntidadeOrganizacao::ORGANIZACAO_INATIVADA_SECUNDARIA;
        } else if($e->isSaas()) {
            $prepStatus = EntidadeOrganizacao::ORGANIZACAO_INATIVADA;
        } else {
            $prepStatus = EntidadeOrganizacao::ORGANIZACAO_INATIVADA_LOCAL_HOSTING;
        }
        if($e->getPrepstatus() != EntidadeOrganizacao::BASE_BLOQUEADA) {
            Util\InstanceProvider::getRepository()->getOrganization()->saveOrganizationPrepStatusBkp($e->getPrepstatus(), $e->getNrorg());
        }
        $e->setPrepstatus($prepStatus);
        $e->setIdativo(EntidadeOrganizacao::INATIVO);
        return $e;
    }

    public function defineStatusBloqueado(EntidadeOrganizacao $e) {
        if($e->getPrepstatus() != EntidadeOrganizacao::ORGANIZACAO_INATIVADA) {
            Util\InstanceProvider::getRepository()->getOrganization()->saveOrganizationPrepStatusBkp($e->getPrepstatus(), $e->getNrorg());
        }
        $e->setPrepstatus(EntidadeOrganizacao::BASE_BLOQUEADA);
        $e->setIdativo(EntidadeOrganizacao::ATIVO);
        return $e;
    }

    public function defineStatusBackup(EntidadeOrganizacao $e) {
        $statusBackup = Util\InstanceProvider::getRepository()->getOrganization()->getOrganizationPrepStatusBkp($e->getNrorg());
        $e->setPrepstatus($statusBackup['PREPSTATUSBKP']);
        $e->setIdativo(EntidadeOrganizacao::ATIVO);
        return $e;
    }

    public function defineStatusBaseRemovida(EntidadeOrganizacao $e) {
        $e->setPrepstatus(EntidadeOrganizacao::ORGANIZACAO_INATIVADA_BASE_REMOVIDA);
        return $e;
    }

    public function defineStatusGerarEstrutura(EntidadeOrganizacao $e) {
        if($e->isSecundaria()) {
            $prepStatus = EntidadeOrganizacao::PREPARACAO_PENDENTE_SECUNDARIA;
        } else if($e->isSaas()) {
            $prepStatus = EntidadeOrganizacao::GERAR_ESTRUTURA;
        } else {
            $prepStatus = EntidadeOrganizacao::PREPARACAO_PENDENTE_LOCAL_HOSTING;
        }
        Util\InstanceProvider::getRepository()->getOrganization()->saveOrganizationPrepStatusBkp($e->getPrepstatus(), $e->getNrorg());
        $e->setPrepstatus($prepStatus);
        $e->setIdativo(EntidadeOrganizacao::ATIVO);
        return $e;
    }

    public function defineNovoStatus(EntidadeOrganizacao $e, $prepStatus) {
        Util\InstanceProvider::getRepository()->getOrganization()->saveOrganizationPrepStatusBkp($e->getPrepstatus(), $e->getNrorg());
        $e->setPrepstatus($prepStatus);
        return $e;
    }

    public function defineNovoStatusBkp(EntidadeOrganizacao $e, $prepStatusBkp) {
        Util\InstanceProvider::getRepository()->getOrganization()->saveOrganizationPrepStatusBkp($prepStatusBkp, $e->getNrorg());
    }

    public function saveScoreOrganization(EntidadeOrganizacao $e, $operador, $score, $alert, $godfather, $manager, $inProject, $director, $responsible, $idaltnrscore) {
        $e->setNrscore($score);
        $e->setIdalerta($alert);
        $e->setDsdiretprclie($director);
        $e->setDsrespticlie($responsible);
        $e->setCdoperpadrinho($godfather);
        $e->setCdopergerente($manager);
        $e->setIdopergerproj($inProject);


        $e->setCdoperultatu($operador);
        $e->setDtultatu(new SetformsDateTime());
        $e->setIdaltnrscore($idaltnrscore);

        return $e;
    }
    
    public function saveNrScore(EntidadeOrganizacao $e, $nrscore) {
        $e->setNrscore($nrscore);
        return $e;
    }
}
