<?php

namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Util\Messages;
use OrganizationsManagement\Util\Utilities;
use Teknisa\Model\Integracao\Repositories as IntegracaoRepositories;
use Teknisa\Model\Geral\Repositories as GeralRepositories;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\Log\Exception;
use Teknisa\Model\Util\NovoCodigo as NovoCodigo;

class StructureUpdate {
    const DOUBLE_BREAK_LINE = '

';

    /**
     * @param $organizationId
     * @param $script
     * @throws \Exception
     * @return array
     */
    public function executeScript($organizationId, $script) {
        $result = '';
        try {
            $entityManager = InstanceProvider::getDBConnection()->getOrgEntityManager($organizationId);
            $scriptResult  = $entityManager->getConnection()->executeQuery($script);
            if($scriptResult->rowCount() > 0) {
                $result = str_replace('{{rowCount}}', $scriptResult->rowCount(), Messages::SUC_ROWS_UPDATED);
            }
            $resultId = "S";
        } catch(\Exception $e) {
            $resultId = "E";
            $ignoredErrors = explode(',', Utilities::getStructureUpdateIgnoredErrorsParameter());
            $result = trim($e->getMessage());
            $isIgnoredError = false;


            foreach($ignoredErrors as $ignoredError) {
                if(strpos($ignoredError, ":")){
                    $expressions = explode(":", $ignoredError);
                    $errorCode = array_shift($expressions);
                    if(strpos($result, $errorCode)){
                        $isIgnoredError = $this->checkIgnoredErrors($result, $ignoredError, true, $expressions);
                    }
                } else{
                    $isIgnoredError = $this->checkIgnoredErrors($result, $ignoredError, false);
                }
                if($isIgnoredError){
                    break;
                }
            }


            if($isIgnoredError) {
                $result .= self::DOUBLE_BREAK_LINE . Messages::SUC_IGNORED_ERROR;
                $resultId = "S";
            }
        }

        return array($result, "result" => $resultId);
    }


    public function checkIgnoredErrors($result, $ignoredError, $hasMnemonic, $expressions = null){
        if($hasMnemonic){
            foreach($expressions as $expression){
                if(strpos(strtoupper($result), strtoupper($expression))){
                    return true;
                }
            }
            return false;
        } else{
            return strpos($result, $ignoredError) ? true : false;
        }
    }


    /**
     * @param $script
     * @param $description
     * @param $logDir
     * @param $organizationIds
     * @param $ticketId
     * @param $fullLogDir
     * @param $logResults
     * @param $fullLogResult
     * @return array
     * @throws \Exception
     */

    public function saveStructureUpdateForOrgs($logDir, $organizationId, $idResultScript, $structureUpdateId, $structureUpdateOrgId) {
        $entityManager = InstanceProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        $atualizacaoEstruturaOrgRepository = InstanceProvider::getRepository()->getOmAtualizacaoEstruturaOrg();

        try {
            $connection->beginTransaction();


            $structureUpdateOrg = $atualizacaoEstruturaOrgRepository->findOneBy(array("id" => $structureUpdateOrgId));


            $structureUpdateOrg->setDsdirlog($logDir);
            $structureUpdateOrg->setNrorgId($organizationId);
            $structureUpdateOrg->setNratuestrutid($structureUpdateId);
            $structureUpdateOrg->setIdresultscript($idResultScript);


            $entityManager->persist($structureUpdateOrg);
            $entityManager->flush();
            $connection->commit();

            return array(
                "ID" => $structureUpdateOrg->getId()
            );
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function saveStructureUpdate($script, $description, $ticketId, $idResultScript, $scriptSuc, $scriptErr, $organizations) {
        $entityManager = InstanceProvider::getEntityManager();
        $environment   = InstanceProvider::getEnvironment();
        $connection    = $entityManager->getConnection();
        $atualizacaoEstruturaFactory = InstanceProvider::getFactory()->getAtualizacaoEstrutura();

        try {
            $structureUpdateOrgIds = [];
            $connection->beginTransaction();

            $userId = $environment->getCurrentUserId();
            $orgId  = $environment->getCurrentOrganizationId();

            $structureUpdate = $atualizacaoEstruturaFactory->save(null, $userId, $orgId, $script, $description, $ticketId, $idResultScript, $scriptSuc, $scriptErr);

            $entityManager->persist($structureUpdate);
            $entityManager->flush();
            foreach($organizations as $org){
                $structureUpdateOrg = $atualizacaoEstruturaFactory->saveOrg(null, $userId, $orgId, $org, $structureUpdate->getId(), "EMPTY", "P");
                array_push($structureUpdateOrgIds, $structureUpdateOrg->getId());
                $entityManager->persist($structureUpdateOrg);
                $entityManager->flush();
            }


            $connection->commit();

            return array(
                "ID" => $structureUpdate->getId(),
                "ORG_IDS" => $structureUpdateOrgIds
            );
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function saveStructureUpdateResults($structureUpdateId, $scriptSuc, $scriptErr){
        $entityManager = InstanceProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        $atualizacaoEstruturaRepository = InstanceProvider::getRepository()->getOmAtualizacaoEstrutura();
        $atualizacaoEstruturaOrgRepository = InstanceProvider::getRepository()->getOmAtualizacaoEstruturaOrg();

        try {
            $orgSuccess = [];
            $orgFails = [];
            $idResultScript = "P";
            $connection->beginTransaction();

            $structureUpdate = $atualizacaoEstruturaRepository->findOneBy(array("id" => $structureUpdateId));

            $structureUpdateOrgs = $atualizacaoEstruturaOrgRepository->findBy(array("nratuestrutid" => $structureUpdateId));


            foreach($structureUpdateOrgs as $structureOrgs){
                if($structureOrgs->getIdresultscript() == "S"){
                    array_push($orgSuccess, "S");
                } else if($structureOrgs->getIdresultscript() == "E"){
                    array_push($orgFails, "E");
                }
            }


            if(sizeof($orgSuccess) && !sizeof($orgFails)){
                $idResultScript = "S";
            } else if(!sizeof($orgSuccess) && sizeof($orgFails)){
                $idResultScript = "E";
            }

            $structureUpdate->setScriptSuc($scriptSuc);
            $structureUpdate->setScriptErr($scriptErr);
            $structureUpdate->setIdresultscript($idResultScript);

            $entityManager->persist($structureUpdate);
            $entityManager->flush();
            $connection->commit();

            return array(
                'ID' => $structureUpdate->getId(),
            );
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }


    /**
     * @param $filters
     * @param $page
     * @param $pageSize
     * @param $orderBy
     * @param $scriptId
     * @return array
     */
    public function findStructureUpdate($filters, $page, $pageSize, $orderBy, $scriptId = null) {
        $conditions = $this->getStructureUpdateFilters($filters, isset($scriptId));
        $strOrder = '';
        foreach($orderBy as $idx => $order) {
            $strOrder .= ' ORDER BY ' . $idx . ' ' . $order;
        }
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $orgId  = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        if($scriptId) {
            $structure = InstanceProvider::getRepository()->getOmAtualizacaoEstrutura()->findStructureUpdateByScriptId($orgId, $scriptId, $conditions, $strOrder, $limit, $offset);
        } else {
            $structure = InstanceProvider::getRepository()->getOmAtualizacaoEstrutura()->findStructureUpdate($orgId, $conditions, $strOrder, $limit, $offset);
        }
        return $structure;
    }

    /**
     * @param $filters
     * @param $scriptId
     * @return string
     */
    private function getStructureUpdateFilters($filters, $scriptId) {
        $structFields    = array (
            'DTINCLUSAO',
            'DSATUESTRUTURA',
            'TICKET_ID',
            'SCRIPT',
            $scriptId ? '' : 'ID',
            $scriptId ? '' : 'IDRESULTSCRIPT'
        );
        $structOrgFields = array (
            'NRORG_ID',
            $scriptId ? 'ID' : '',
            $scriptId ? 'IDRESULTSCRIPT' : ''
        );
        $structFilters = $structOrgFilters = $operatorFilters = array();
        $likeAllFilter = '';

        foreach($filters as $filter) {
            $filter['columnName'] = str_replace('ORGANIZATION_IDS', 'NRORG_ID', $filter['columnName']);
            $filter['columnName'] = str_replace('ORGANIZATION_ID', 'NRORG_ID', $filter['columnName']);
            $filter['columnName'] = str_replace('STRUCT_ID', 'ID', $filter['columnName']);
            if($filter['operator'] == 'MAPPED_LIKE_ALL') {
                $value = "UPPER('" . $filter['value'][0]['_ALL'] . "')";;
                $columnList = explode('|', explode('#', $filter['columnName'])[0]);
                foreach($columnList as $column) {
                    $tableAlias = 'OPE';
                    if(in_array($column, $structFields)) {
                        $tableAlias = 'OAE';
                    } else if(in_array($column, $structOrgFields)) {
                        $tableAlias = 'OAO';
                    }
                    $likes[] = "UPPER($tableAlias.$column) LIKE $value";
                    $likeAllFilter = "AND ( " . implode($likes, ' OR ') . " ) ";
                }
            } else {
                if(in_array($filter['columnName'], $structFields)) {
                    array_push($structFilters, $filter);
                } else if(in_array($filter['columnName'], $structOrgFields)) {
                    array_push($structOrgFilters, $filter);
                } else {
                    array_push($operatorFilters, $filter);
                }
            }
        }

        $structFilters    = Utilities::createQueryFiltered($structFilters,    'OAE');
        $structOrgFilters = Utilities::createQueryFiltered($structOrgFilters, 'OAO');
        $operatorFilters  = Utilities::createQueryFiltered($operatorFilters,  'OPE');

        return $structFilters . $structOrgFilters . $operatorFilters. $likeAllFilter;
    }

    /**
     * @param $fileDir
     * @param $fileName
     */
    public function downloadStructureUpdateLog($fileDir, $fileName) {
        Utilities::downloadAndRemoveFile(Utilities::getFileServerFilesUrlParameter() . $fileDir, $fileName, 'text/plain', false);
    }

    public function downloadStructureUpdateScripts($fileName, $fileDir, $ticket){
        $allScriptsSuccess = $this->findSuccessScripts($ticket);
        $filePath = $fileDir. "/" . $fileName;

        file_put_contents($filePath, $allScriptsSuccess);
        Utilities::downloadAndRemoveFile($filePath, $fileName, 'text/plain', true);
    }


    public function validateSucceededScripts($ticket){
        $omAtualizacaoEstruturaRepository = InstanceProvider::getRepository()->getOmAtualizacaoEstrutura();
        $omAtualizacaoEstruturas = $omAtualizacaoEstruturaRepository->findBy(array('ticketId' => $ticket), array('id' => 'ASC'));
        $isSuccess = false;

        foreach($omAtualizacaoEstruturas as $oma){
            if($oma->getIdresultscript() == 'S'){
                $isSuccess = true;
            }
        }
        return $isSuccess;
    }

    public function findErrorOrPartialOrganizations($id, $result){
        $errorOrgs = InstanceProvider::getRepository()->getOmAtualizacaoEstrutura()->findErrorOrPartialOrganizations($id, $result);
        return $errorOrgs;
    }

    protected function findSuccessScripts($ticket){
        $omAtualizacaoEstruturaRepository = InstanceProvider::getRepository()->getOmAtualizacaoEstrutura();
        $omAtualizacaoEstruturas = $omAtualizacaoEstruturaRepository->findBy(array('ticketId' => $ticket), array('id' => 'ASC'));
        $scriptsSuccess = null;
        $allScriptsSuccess = [];


        foreach($omAtualizacaoEstruturas as $oma){
            if($oma->getScriptSuc() && !in_array($oma->getScriptSuc(), $allScriptsSuccess)){
                if($scriptsSuccess){
                    $scriptsSuccess .=  "\n/\n" . $oma->getScriptSuc();
                } else if($oma->getScriptSuc()){
                    $scriptsSuccess = $oma->getScriptSuc();
                }
                array_push($allScriptsSuccess, $oma->getScriptSuc());
            }
        }
        return $scriptsSuccess;
    }

    
    public function validateTicket($ticket) {
        $valid = InstanceProvider::getRepository()->getContato()->validateTicket($ticket);
        return $valid[0]["COUNT"] == "0" ? false : true;
    }

    public function validateTicketIfScriptAlreadySucceeded($ticket){
        $allScriptsSuccess = $this->findSuccessScripts($ticket);

        if(!$allScriptsSuccess){
            return false;
        } else{
            return true;
        }
    }
}